const shell = require('shelljs')
const fs = require('fs')
const path = require('path')
const zipper = require('zip-local')
const envDefine = require('./build-en-define.json')
const md5 = require('md5')
const sha256 = require('./sha256')
const clc = require('./clc')

const argv = process.argv.splice(2, 1)[0]
if (!argv) {
  clc.red('未指定插件名')
  process.exit(0)
}
const crypto = process.argv.splice(2, 1)[0] === '-c'
const noZip = process.argv.splice(2, 1)[0] === '-Z'
const env = process.argv.splice(2, 1)[0]
const realEnv = (env && env.split(',')) || envDefine
const outputSha256 = process.argv.splice(2, 1)[0] === 'sha'

const lib = async (cmd, program) => {
  const customModule = cmd.custom
  // 获取指定 custom 目录下的apaas.json
  const customModulePath = path.resolve(process.cwd(), 'src/custom', customModule)

  const customModuleConfigPath = path.resolve(customModulePath, 'apaas.json')
  if (!fs.existsSync(customModuleConfigPath)) {
    clc.red(`Error: 自开发模块 ${customModule} 下不存在需要的 apaas.json 文件`)
    clc.red('exit')
    process.exit(0)
  }

  const apaasConfig = JSON.parse(fs.readFileSync(customModuleConfigPath))
  if (apaasConfig.extend) {
    const extendModulePath = `${customModulePath}/${apaasConfig.extend}`
    fs.existsSync(extendModulePath) && shell.exec(`node ${extendModulePath}`)
  }

  const buildArray = realEnv
    .map((_) => {
      if (!apaasConfig[_]) {
        return
      }
      const newConfig = { ...apaasConfig }
      delete newConfig[_]
      newConfig.outputName = md5(apaasConfig.code)
      return {
        // 排第一，且不会被覆盖
        entry: apaasConfig[_],
        ...newConfig,
        entry: apaasConfig[_],
        type: _
      }
    })
    .filter((item) => item)
  clc.cyan(`Building ${customModule} (${apaasConfig.code})...`)
  const result = buildArray.map((config) => doLib(config, customModulePath))
  if (result === [-1, -1]) {
    process.exit(0)
  }
  const cryptoPath = crypto ? 'crypto/' + md5(apaasConfig.code) : apaasConfig.code
  writeApaasJson(cryptoPath, apaasConfig)
  if (!noZip) {
    zip(cryptoPath, cryptoPath, customModule)
  }
  clc.green(`Built ${customModule} (${apaasConfig.code}) successfully!`)
}

/**
 * @params apaasModuleConfig 源配置信息
 * @params customModulePath 打包入口
 */
const doLib = (apaasModuleConfig, customModulePath) => {
  const customModuleEntryPath = path.resolve(customModulePath, apaasModuleConfig.entry)

  const moduleType = apaasModuleConfig.type

  const outputName =
  (crypto ? 'crypto/' + md5(apaasModuleConfig.code) : apaasModuleConfig.code) + '/' + moduleType

  if (!fs.existsSync(customModuleEntryPath)) {
    clc.red(`Error: apaas.json 指定的entry: ${apaasModuleConfig.entry} 的路径错误 `)
    clc.red(`error path is ${customModuleEntryPath}`)
    return -1
  }
  clc.magenta(`Building ${moduleType} of ${apaasModuleConfig.code}...`)

  const customCliPath = path.resolve(
    process.cwd(),
    './node_modules/@vue/cli-service/bin/vue-cli-service.js'
  )

  // 使用vue build 打包，并生成文件
  shell.exec(
    `node ${customCliPath} build --target lib --name ${apaasModuleConfig.outputName} --dest ${outputName} ${customModuleEntryPath} --formats umd,umd-min`
  )

  const suffix = ['umd.js', 'umd.min.js', 'css']
  suffix.forEach((item) => {
    shell.mv(
      path.resolve(process.cwd(), `${outputName}/${apaasModuleConfig.outputName}.${item}`),
      path.resolve(process.cwd(), `${outputName}/index.${item}`)
    )
  })
  // shell.cp('-R', customModuleConfigPath, `${outputPath}/`)

  const newOutputConfig = { ...apaasModuleConfig }

  realEnv.forEach((_) => {
    delete newOutputConfig[_]
  })

  const outputPath = path.resolve(process.cwd(), outputName)

  // rm demo.html
  shell.rm(path.resolve(process.cwd(), `${outputName}`, 'demo.html'))

  // 拷贝public文件到指定目录
  apaasModuleConfig.copyAssets.forEach((copyAsset) => {
    copyAssetSource = copyAsset + copyAsset.endsWith('/') ? moduleType : '/' + moduleType
    const assetPath = path.resolve(process.cwd(), copyAssetSource)
    if (fs.existsSync(assetPath)) {
      const outputAsset = path.resolve(outputPath, copyAsset.replace('public/', 'static/'))
      shell.mkdir('-p', outputAsset)
      shell.cp('-R', `${assetPath}/*`, `${outputAsset}/`)
    }
  })
  clc.yellow(`Built ${moduleType} of ${apaasModuleConfig.code} successfully!`)
}

/**
 * @params outputPath 目标文件夹命
 * @params originJson 源配置信息
 */
const writeApaasJson = function(outputPath, originJson) {
  const newOutputConfig = { ...originJson }
  delete newOutputConfig.type
  delete newOutputConfig.outputName
  delete newOutputConfig.local
  newOutputConfig.extraConfig = { ...newOutputConfig.extraConfig }
  realEnv.forEach((item) => {
    if (newOutputConfig[item]) {
      newOutputConfig.extraConfig[item] = true
      delete newOutputConfig[item]
      newOutputConfig.extraConfig[item + 'Css'] = fs.existsSync(
        path.resolve(process.cwd(), outputPath, item, 'index.css')
      )
      const md5Code = md5(fs.readFileSync(path.resolve(`${outputPath}/${item}/index.umd.min.js`)))
      newOutputConfig.extraConfig[item + 'Version'] = md5Code
    }
  })
  fs.writeFile(`${outputPath}/apaas.json`, JSON.stringify(newOutputConfig, null, '  '), function(
    error
  ) {
    if (error) {
      clc.red(`配置信息写入失败\n${error}`)
    }
  })
}

/**
 * @params originPath 源文件夹名
 * @params outputName 目标文件名
 */
const zip = function(originName, outputName, customModule) {
  const originPath = path.resolve(process.cwd(), originName)
  const outputZipPath = path.resolve(process.cwd(), `${outputName}.zip`)
  if (fs.existsSync(outputZipPath)) {
    shell.rm(outputZipPath)
  }
  // 生成zip并压缩
  zipper.zip(originPath, function(error, zipped) {
    if (!error) {
      zipped.compress() // compress before exporting

      var buff = zipped.memory() // get the zipped file as a Buffer

      zipped.save(outputZipPath, function(error) {
        if (!error) {
          clc.green('zipped successfully !')
          if (outputSha256) {
            sha256.generate(customModule, outputZipPath)
          }
          // 删除指定生成目录
          // shell.rm('-r',originPath)
        } else {
          clc.red(`write error: ${error}`)
        }
      })
    } else {
      clc.red(`read error: ${error}`)
    }
  })
}

module.exports = lib(
  {
    custom: `plugin-${argv}`
  },
  {}
)
